// PROGRAM: genea
// STARTING DATE: 13-11-95
// FINISHING DATE: August '96
// AUTHOR: Koen van der Hauw
// Implementation of an Evolutionary Algorithm version written 
// for experimentation for Master's Thesis: "Evaluating and Improving 
// Steady State Evolutionary Algorithms on Constraint Satisfaction Problems."
// 
// In the current version the Graph 3-Coloring problem is implemented.

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <sys/time.h>
#include <math.h>
#include <string.h>
#include "typedef.h"
#include "problem.h"

// possible operators for crossover and mutation 
enum Operator {noop, onepoint, twopoint, mpoint, uniform, hux, diag,
      scun, edge, order, order2, pmx, cycle, 
      normal, intmut, heur, inversion, swap, rar};
#define OVERCROSSOVER uniform // Operator used for recombination
#define MUTATION intmut      // Operator used for mutation
#define MR 0               // Mutation Rate. 0 means 1/L
#define MAXPOPSIZE 200       // Number of chromosomes in population
#define MAXCROSSPOINTS 34  // nr of crossover points for Mpoint crossover
#define MAXPARENTS 2       // Maximal nr of parents for MParent operators
#define OPUSPERGEN 1       // Number of operator uses per generation
#define MAXOFFSPRING 2     // Maximal nr of offspring
#define PRESERVATIVE true  // Preservative or Extinctive Selection
#define TOURNSIZE 2        // tournament size for parent selection
#define INTREP true       // Integer or Order Representation

// possible stopcriteria 
enum StopCriteria { noconv, fitdifconv, fitvarconv, genevarconv, entropyconv,
                    allconv};
#define STOPCRIT noconv    // Stopcriterium. To be effective the
// pop.ComputeConvergence(evals); - statement should be uncommented
#define MAXCONVTIME 0      // Maximum nr. of evaluations in a converged pop.

// SAW Parameters
enum Weights {nowghts, degreewghts, sawwghts};
#define WEIGHTS sawwghts
#define WGHTCHEVAL 250       // Only used when WEIGHT = sawwghts
#define WGHTINC 1            // idem

// Incest Prevention parameters
#define INCESTPREV true  // Works only for integer Representation
#define NROFFAILS 10      // number of times no incestfree parents were found

#define N         200     // Problem Size
#define RNDSEED   5        // Seed for instance generator
#define PFROM     0.025     // Edge probability to start with in experiment
#define PTO       PFROM    // Edge probability to end with in experiment
#define PSTEP     0.005    // Stepsize in edge probability loop
#define POPSIZEFROM MAXPOPSIZE // Starting population size in loop
#define POPSIZESTEP 50    // Population stepsize in experimentation loop

#define MAXEVALS 50000    // Maximum number of Evaluations for each run
#define NROFRUNS 10        // Number of times the GA is run independently

#define WRITEFILEINTERVAL 100     // Nr of evaluations between report to file
#define RESULTDIR "../results/"  // Directory to write results to
#define FILELABEL "AssexOrdSAW"          // Unique label for reportfiles
#define REMOVETMPFILES false         // Remove intermediate reportfiles
#define EFECERTAINTY 0.9          // Certainty of EFE, so value of alpha


/* Global variables */
MLCG rnd;              /* class to generate random numbers */
int nrofgenes;         /* Number of genes for current problem */
Operator crossover;    /* current operator */
int longsize;
int popsize;           /* number of individuals in population */
double invpopsize;     /* 1/popsize, to save a divide */
int nrofparents;       /* number of parents for current operator */
int nrofoffspring;     /* number of offspring for current operator */
int crosspoints;       /* number of crossoverpoints for m-point crossover */
char methodname[MAXPATHLENGTH]; /* includes operator used */
ofstream reportoutput;   /* file where final report in put into */
double totalevaltime;    /* total over all runs of time per evaluation */

/* Gives time in seconds to use as a seed number */
int getsec() {
  
  struct timeval tp;
  struct timezone tz;

  gettimeofday(&tp, &tz);
  return tp.tv_usec;
  
} /* getsec() */

clock_t starttime,
     stoptime;

void StartTimer() {
 
  starttime = clock();

} /* StartTimer() */

void StopTimer() {

  stoptime = clock();

} /* StopTimer() */

double GetRunTime() {

  return ((double) (stoptime - starttime) / CLOCKS_PER_SEC);

} /* RetRunTime() */

// Class definitions


typedef int Allel;

class GenoType {
public: 
  Allel gene[MAXGENES];
  double fitness;
  GenoType() {}
  void IntegerInitialize();
  void OrderInitialize();
  void SetAllel(int, int);
  void IntegerEvaluate();
  void OrderEvaluate();
  void MutateInteger();
  void MutateNormal();
  void MutateHeur();
  void MutateHillClimb();
  void Mutate1();
  void MutateM();
  void MutateInversion();
  void MutateSwap();
  void MutateRAR();
  void Copy(GenoType&);
  void Crossover1(GenoType&, GenoType&, GenoType&);   
  void Crossover2(GenoType&, GenoType&, GenoType&);   
  void CrossoverM(GenoType&, GenoType&, GenoType&);   
  void CrossoverU(GenoType&, GenoType&, GenoType&);
  void HUX(GenoType&, GenoType&, GenoType&);
  void CrossoverDIAGMP(int[], int, GenoType[], int);
  void CrossoverSCUNMP(int[], int, GenoType[], int);
  void CrossoverEdge(int[], int, GenoType[], int);
  void CrossoverOrder(GenoType&, GenoType&, GenoType&);
  void CrossoverOrder2(GenoType&, GenoType&, GenoType&);
  void CrossoverPMX(GenoType&, GenoType&, GenoType&);
  void CrossoverCycle(GenoType&, GenoType&, GenoType&);
  void Print();
};


/* Initialize a chromosome with random values from its domain */
void GenoType::IntegerInitialize() {

  int i;
  Link* ptr;

  for (i = 0; i < nrofgenes; i++) {
    gene[i] = rnd.asLong() % NROFCOLORS;
    for (ptr = problem.gradjl[i]; ptr; ptr = ptr->nextptr)
          ptr->weight = 1;    
  }

} /* GenoType::IntegerInitialize() */
  

/* Initialize a chromosome with random permutation */
void GenoType::OrderInitialize() {

  int i, j;
  int nodesleft;
  int begin;
  int end;
  int index;
  int next[MAXGENES];
  int prev[MAXGENES];

  /* Initialize list */
  for (i = 0; i < nrofgenes; i++) {
    next[i] = i+1;
    prev[i] = i-1;                                  
  }                                                 
  begin = 0;                                        
  end = nrofgenes-1;                                
  nodesleft = nrofgenes;                            
                                                    
  /* Create Permutation of nodes */                 
  for (i = 0; i < nrofgenes; i++, nodesleft--) {    
    /* Pick a node amongst the left ones */         
    j = (rnd.asLong() % nodesleft) + 1;  
    /* search picked node */                        
    if (j > nodesleft - j)                    
      for (index = end, j = nodesleft-j; j > 0; index = prev[index], j--);
    else                                            
      for (index = begin; j > 1; index = next[index], j--);   
                                                    
    gene[i] = index;                              
                                                    
    /* Remove node from left-list */                
    if (index == end)                               
      end = prev[end];                              
    else                                            
      if (index == begin)                           
        begin = next[begin];                        
      else {                                        
        prev[next[index]] = prev[index];            
        next[prev[index]] = next[index];
      }
  }

} /* GenoType::OrderInitialize() */


/* Set the value of gene 'gennr' to newcolor and ReEvaluate the */
/* chromosome's fitness, using the fact the only gene 'gennr'   */
/* got a (possible) new color 'newcolor'                        */
void GenoType::SetAllel(int gennr, int newcolor) {

  Link *ptr;

  if (gene[gennr] != newcolor) {
    for (ptr = problem.gradjl[gennr]; ptr; ptr = ptr->nextptr) 
      if (gene[gennr] == gene[ptr->nodenr]) {
        fitness -= ptr->weight;
      }
      else
        if (gene[ptr->nodenr] == newcolor) {
          fitness += ptr->weight;
	}
    gene[gennr] = newcolor;
  }

} /* GenoType::SetAllel() */

/* Integer Representation Evaluation Function                        */
/* Compute a chromosome's fitness value                              */
/* As a first simple measure the number of not satisfied constraints */
void GenoType::IntegerEvaluate() {

  int i;
  Link *ptr;
  
  fitness = 0;
  for (i = 0; i < nrofgenes; i++)
    for (ptr = problem.gradjl[i]; ptr; ptr = ptr->nextptr)
      if (i > ptr->nodenr)
	if (gene[i] == gene[ptr->nodenr]) {
	  fitness += ptr->weight;    
        }

} /* GenoType::IntegerEvaluate() */

#define UNCOLORED -1

/* Order Based Representation Evaluation Function      */
/* Compute a chromosome's fitness value                */
/* Use greedy algorithm to compute fitness of ordering */
void GenoType::OrderEvaluate() {

  int color[MAXGENES];
  int i, j;
  int index;
  bool avail[MAXGENES][NROFCOLORS];
  Link* ptr;

  /* Initialize */
  for (i = 0; i < nrofgenes; i++) {
    color[i] = UNCOLORED;
    for (j = 0; j < NROFCOLORS; j++)
      avail[i][j] = true;
  }

  /* Color all nodes */
  fitness = 0;
  for (i = 0; i < nrofgenes; i++) {
    index = gene[i];
    for (j = 0; (j < NROFCOLORS) && !avail[index][j]; j++);
    if (j < NROFCOLORS) {
      color[index] = j;

      for (ptr = problem.gradjl[index]; ptr; ptr = ptr->nextptr)
        if (color[ptr->nodenr] == UNCOLORED)
          avail[ptr->nodenr][j] = false;
    }
    else {
	switch(WEIGHTS) {
        case nowghts: fitness++;
                      break;
        case degreewghts: fitness += problem.nrofneighbors[index];
                          break;
        case sawwghts: fitness += problem.weight[index];
                       break;
        default: cout << "No Weight Mechanism Defined!" << endl;
      }
    }
  }
  /*fitness = 0;
  for (i = 0; i < nrofgenes; i++)
    for (ptr = problem.gradjl[i]; ptr; ptr = ptr->nextptr)
      if (i > ptr->nodenr)
	if (color[i] == color[ptr->nodenr]) 
	  fitness += ptr->weight;    */

  //fitness++;  //voor als recombination getest moet worden...
  /*if (fitness == 0)  {
    for (i = 0; i < nrofgenes; i++)
      cout << color[i];
    cout << endl;
  }*/

} /* GenoType::OrderEvaluate() */


/* Normal/Classical mutation: use mutation rate = 1/chromosomelength */
void GenoType::MutateNormal() {

  double mr = 1.0 / (double) nrofgenes;
  int i;

  for (i = 0; i < nrofgenes; i++) {
    if (rnd.asFloat() < mr)
      SetAllel(i, rnd.asLong() % NROFCOLORS);
  }

} /* GenoType::MutateNormal() */

const int PROBPREC = 10;          // Precision of nr of genes prob is computed
double genmutcumprob[PROBPREC+1]; // Cumulative probs. for normal mutation 

double Prob(int n, int r) {

  double mr;
  double res;
  int i;

  if (MR)
    mr = MR;
  else
    mr = 1.0 / (double) n;
  
  res = 1;
  for (i = 1; i <= r; i++) 
    res = ((res*(n-i+1)) / i) * mr;
  res *= pow (1.0 - mr, n - r);

  return res; 

} /* Possibilities() */

void CreateGenMutProbs() {

  int i;

  genmutcumprob[0] = 0;  
  for (i = 1; i < PROBPREC; i++) 
    genmutcumprob[i] = genmutcumprob[i-1] + Prob(nrofgenes, i-1);
  genmutcumprob[PROBPREC] = 1.0;

} /* CreateGenMutProbs() */


/* Fast version of normal mutation, uses in advanced computed */
/* probability distribution to compute nr of genes mutated.   */
void GenoType::MutateInteger() {

  double mgs;   
  int genes[PROBPREC];
  int i, j;

  mgs = rnd.asFloat();

  // Make sure that distinct genes are mutated 
  // BUG: genes sometimes aren't distinct!!    
  for (i = 0; mgs > genmutcumprob[i+1]; i++) {
    genes[i] = rnd.asLong() % (nrofgenes - i);
    for (j = 0; j < i; j++) 
      if (genes[i] >= genes[j])
        genes[i]++;
    SetAllel(genes[i], rnd.asLong() % NROFCOLORS);
    // (gene[i]+(rnd.asLong()%(NROFCOLORS-1)))%NROFCOLORS); 
  } 

//  SetAllel(genes[rnd.asLong() % nrofgenes], rnd.asLong() % NROFCOLORS);
  
  
} /* GenoType::MutateInteger() */


void GenoType::MutateHeur() {

  double mgs;   
  int genes[PROBPREC];
  int i, j;
  double maxfit,
         oldfit;
  int maxval[NROFCOLORS],
      nrofmax,
      oldval;

  mgs = rnd.asFloat();

  /* Make sure that distinct genes are mutated */
  for (i = 0; mgs > genmutcumprob[i+1]; i++) {
    genes[i] = rnd.asLong() % (nrofgenes - i);
    for (j = 0; j < i; j++) 
      if (genes[i] >= genes[j])
        genes[i]++;

    maxfit = oldfit = fitness;
    maxval[0] = 0;
    nrofmax = 1;  
    oldval = gene[genes[i]];
    for (j = 1; j < NROFCOLORS; j++) {
      SetAllel(genes[i], (oldval + j) % NROFCOLORS);
      if (fitness < maxfit) {
        maxfit = fitness;
        nrofmax = 1;
        maxval[0] = j;
      }
      else
        if (fitness == maxfit) {
          maxval[nrofmax] = j;
          nrofmax++;
        }
    }
    if (nrofmax > 1) {
      nrofmax = rnd.asLong() % nrofmax;
      SetAllel(genes[i], (oldval + maxval[nrofmax]) % NROFCOLORS);
    }
    else
      if (maxval[0] != j-1)
        SetAllel(genes[i], (oldval + maxval[0]) % NROFCOLORS);
  } 

} /* GenoType::MutateHeur() */

void GenoType::MutateInversion() {

  int g1, g2, i;
  Allel temp;

  if (rnd.asLong() & 1) {
    g1 = rnd.asLong() % nrofgenes;
    g2 = rnd.asLong() % nrofgenes; 

    if (g1 < g2)
      for (i = 0; i < (g2 - g1+1)/2; i++) {
         temp = gene[g1 + i];
         gene[g1 + i] = gene[g2 - i];
         gene[g2 - i] = temp;
      }  
    else
      for (i = 0; i <= (g2 + 1 + nrofgenes - g1)/2; i++) {
        temp = gene[(g1 + i) % nrofgenes];
        gene[(g1 + i) % nrofgenes] = gene[(g2+nrofgenes-i) % nrofgenes];
        gene[(g2+nrofgenes-i) % nrofgenes] = temp;
      } 
  }

} /* GenoType::MutateInversion() */

void GenoType::MutateSwap() {

  double mgs;   
  int i, g1, g2;
  Allel temp;

  mgs = rnd.asFloat();

  // Perform number of swaps 
  for (i = 0; mgs > genmutcumprob[i+1]; i++) {
    g1 = rnd.asLong() % nrofgenes;
    g2 = rnd.asLong() % nrofgenes;
    temp = gene[g1];
    gene[g1] = gene[g2];
    gene[g2] = temp;
  }

/*  g1 = rnd.asLong() % nrofgenes;
  g2 = rnd.asLong() % nrofgenes;
  temp = gene[g1];
  gene[g1] = gene[g2];
  gene[g2] = temp; */

} /* GenoType::MutateSwap() */

void GenoType::MutateRAR() {

  double mgs;   
  int i, g1, g2;
  Allel temp;

  mgs = rnd.asFloat();

  /* Perform number of reinsertions */
  for (i = 0; mgs > genmutcumprob[i+1]; i++) {
    g1 = rnd.asLong() % nrofgenes;
    g2 = rnd.asLong() % nrofgenes;
    temp = gene[g1];
    if (g1 <= g2)
      for (i = g1; i < g2; i++)
        gene[i] = gene[i+1];
    else {
      for (i = g1; i < nrofgenes-1; i++) 
        gene[i] = gene[i+1];
      gene[nrofgenes-1] = gene[0];
      for (i = 0; i < g2; i++)
        gene[i] = gene[i+1];
    }
    gene[g2] = temp;
  }

} /* GenoType::MutateRAR() */



// Mutate exactly one gene to new value
void GenoType::Mutate1() {

  int genepos = rnd.asLong() % nrofgenes;

  SetAllel(genepos, (gene[genepos]+(rnd.asLong()%(NROFCOLORS-1)))%NROFCOLORS); 

} /* GenoType::Mutate1() */

/* Mutate maximal 1/4th of chromosome */
void GenoType::MutateM() {

  int i;
  int mutnr = rnd.asLong() % (int) (nrofgenes * 0.25);

  for (i = 0; i < mutnr; i++) 
    SetAllel(rnd.asLong() % nrofgenes, rnd.asLong() % NROFCOLORS);

} /* GenoType::MutateM() */    

/* Just copy a chromosome's genes to another chromosome */
void GenoType::Copy(GenoType& child) {

  int i;

  for (i = 0; i < nrofgenes; i++) {
    child.gene[i] = gene[i];
//    child.nrofviolations[i] = nrofviolations[i];
  }
  child.fitness = fitness;

} /* GenoType::Copy() */

/* Form a child using 1-crossover between two parents. Exchange */
/* genes up to (and including) crossoverpoint with other parent */
void GenoType::Crossover1(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int i;
  int cp;     /* Crossover point */

  /* Choose crossoverpoint with possibility of no crossover */
  /* Make sure at least one gene will be exchanged.         */
  cp = rnd.asLong() % nrofgenes;

  Copy(child1);
  otherparent.Copy(child2);
 
  for (i = 0; i <= cp; i++) {
    child1.SetAllel(i, otherparent.gene[i]);   
    child2.SetAllel(i, gene[i]);
  }
  
} /* GenoType::Crossover1() */

/* Form a child using 2-crossover between two parents. Exchange    */
/* genes between (and including) crossover points between parents. */
void GenoType::Crossover2(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int i;
  int cp1, cp2;     /* Crossover points */

  /* Select two random (not necessarily distinct) crossover points */
  /* with cp1 <= cp2                                               */
  cp1 = rnd.asLong() % nrofgenes;
  cp2 = rnd.asLong() % nrofgenes;
  if (cp2 < cp1) {
    i = cp1;
    cp1 = cp2;
    cp2 = i;
  }

  Copy(child1);
  otherparent.Copy(child2);
  
  for (i = cp1; i <= cp2; i++) {
    child1.SetAllel(i, otherparent.gene[i]);
    child2.SetAllel(i, gene[i]);
  }
				
} /* GenoType::Crossover2() */

/* Form a child using M-crossover between two parents. Exchange    */
/* genes between crossover points between parents.                 */
void GenoType::CrossoverM(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int i, j, temp;
  int cp[MAXCROSSPOINTS+2];     /* Crossover points */

  /* select 'crosspoints' (not DISTINCT!) crossoverpoints */
  cp[0] = 0;
  for (i = 1; i <= crosspoints; i++)
    cp[i] = rnd.asLong() % (nrofgenes);
  cp[crosspoints+1] = nrofgenes;

  /* sort crossoverpoints using insertion-sort */
  for (i = 2; i <= crosspoints; i++) {
    j = i;
    temp = cp[j];
    while (j > 1 && cp[j-1] > temp) {
      cp[j] = cp[j-1];
      j--;
    }
    cp[j] = temp;
  }

  Copy(child1);
  otherparent.Copy(child2);

  for (i = 0; i <= crosspoints; i++)
    for (j = cp[i]; j < cp[i+1]; j++) 
      if (i & 1) {
        child1.SetAllel(j, otherparent.gene[j]);   
        child2.SetAllel(j, gene[j]);
      } 

} /* GenoType::CrossoverM() */

/* Form a child using uniform-crossover between two parents. Choose */
/* each gene randomly from one of the parents.                      */
void GenoType::CrossoverU(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int i,j;
  int longsize;
  long mask;
  
  longsize = sizeof(long) * 8;

  Copy(child1);
  otherparent.Copy(child2);
  
  for (i = 0, j = 0; i < nrofgenes; i++, j--) {
    if (!j) {
      mask = rnd.asLong();
      j = longsize;
    }
    if (mask & 1) {
      child1.SetAllel(i, otherparent.gene[i]);
      child2.SetAllel(i, gene[i]);
    }
    mask >>= 1;
  }

} /* GenoType::CrossoverU() */

/* Highly disruptive crossover from the CHC-algorithm from Eshelman.      */
/* Ensures that always half of the  differing genes will be crossed over. */
void GenoType::HUX(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int posind[MAXGENES];         /* real index of ith different gene */
  int prevind[MAXGENES];        /* to implement linked list with an array */
  int nextind[MAXGENES];
  int i, p,
      d,
      hd,
      rndval,
      firstind;                 /* index of first element in linked list */

  firstind = 0;
  /* find positions of differing genes */
  for (i = 0, d = 0; i < nrofgenes; i++)
    if (gene[i] != otherparent.gene[i]) {
      posind[d] = i; 
      prevind[d] = d-1;
      nextind[d] = d+1;
      d++;
    }
  Copy(child1);
  otherparent.Copy(child2);

  /* crossover exactly half of differing genes */  
  for (hd = d/2; d > hd; d--) {
    rndval = rnd.asLong() % d;
    /* could be possible/better to start from other end ... */
    for (p = firstind, i = 0; i < rndval; p = nextind[p], i++);
    child1.SetAllel(posind[p], otherparent.gene[posind[p]]);
    child2.SetAllel(posind[p], gene[posind[p]]);
    nextind[prevind[p]] = nextind[p];
    prevind[nextind[p]] = prevind[p];
    if (p == firstind)
      firstind = nextind[p];
  }
          
} /* GenoType::Hux() */

/* Diagonal Crossover with multiple parents.                         */
/* Select 'nrofparents-1' crossoverpoints to produce 'nrofoffspring' */
/* children from 'nrofparents' parents using diagonal crossover.     */
/* Ensured is that each child will have at least one gene of         */
/* each parent.                                                      */
void GenoType::CrossoverDIAGMP(int pi[], int parent1, GenoType pool[], 
                           int child1) {

  int i, j, l;
  int cp[MAXPARENTS+1];
  int temp;

  /* select 'nrofparents-1' (not DISTINCT!) crossoverpoints */
  cp[0] = 0;
  for (i = 1; i < nrofparents; i++) 
    cp[i] = rnd.asLong() % (nrofgenes);
  cp[nrofparents] = nrofgenes;

  /* sort crossoverpoints using insertion-sort */
  for (i = 2; i < nrofparents; i++) {
    j = i;
    temp = cp[j];
    while (j > 1 && cp[j-1] > temp) {
      cp[j] = cp[j-1];
      j--;
    }
    cp[j] = temp;
  }
      
  for(i = 0; i < nrofoffspring; i++)
    pool[parent1+i].Copy(pool[child1+i]);  

  /* copy genes upto (and not including) each crossover point */
  for (i = 0; i < nrofparents; i++)  /* crossoverpoints */
    for (j = 0; j < nrofoffspring; j++)   /* childnr */
     for (l = cp[i]; l < cp[i+1]; l++)   /* geneposition */
       pool[child1 + j].SetAllel(l, 
                         pool[pi[parent1 + ((j+i) % nrofparents)]].gene[l]); 

} /* GenoType::CrossoverDIAGMP() */

/* Uniform Scanning Crossover with multiple parents */
void GenoType::CrossoverSCUNMP(int pi[], int parent1, GenoType pool[], 
                           int child1) {
  int i;

  Copy(pool[child1]);
  
  /* choose the gene randomly from one of the parents */
  for (i = 0; i < nrofgenes; i++)
    pool[child1].SetAllel(i, pool[pi[parent1 + rnd.asLong() % nrofparents]].gene[i]);
      
} /* GenoType::CrossoverSCUNMP() */


void GenoType::CrossoverOrder(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  bool avail1[MAXGENES];
  bool avail2[MAXGENES];
  int g1, g2, i;
  int c1, c2;

  for (i = 0; i < nrofgenes; i++) {
    avail1[i] = true;
    avail2[i] = true;
  }

  g1 = rnd.asLong() % nrofgenes;
  g2 = rnd.asLong() % nrofgenes;
  //Print();
  //otherparent.Print();
  //cout << "g1 = " << g1 << ", g2 = " << g2 << endl;

  for (i = g1; i != g2; i = (i + 1) % nrofgenes) {
    child1.gene[i] = otherparent.gene[i];
    avail1[otherparent.gene[i]] = false;
    child2.gene[i] = gene[i];
    avail2[gene[i]] = false;
  }
  child1.gene[g2] = otherparent.gene[g2];
  avail1[otherparent.gene[g2]] = false;
  child2.gene[g2] = gene[g2];
  avail2[gene[g2]] = false;
  
  c1 = g2;
  c2 = g2;
  for (i=(i+1) % nrofgenes; i != g1; i = (i+1) % nrofgenes) {  
    do {
      c1 = (c1 + 1) % nrofgenes;
    } while (!avail1[gene[c1]]);
    child1.gene[i] = gene[c1];
    do {
      c2 = (c2 + 1) % nrofgenes;
    } while (!avail2[otherparent.gene[c2]]);
    child2.gene[i] = otherparent.gene[c2];
  }

  //child1.Print();
  //child2.Print();

} /* GenoType::CrossoverOrder() */


void GenoType::CrossoverOrder2(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int i, j;
  long mask;
  bool avail1[MAXGENES];
  bool avail2[MAXGENES];
  int notdoneyet[MAXGENES];
  int nrofnotdone;
  int c1, c2;

  //Print();
  //otherparent.Print();
    
  nrofnotdone = 0;
  /* select by flipping a coin... */
  for (i = 0, j = 0; i < nrofgenes; i++, j--) {
    if (!j) {
      mask = rnd.asLong();
      j = longsize;
    }
    if (mask & 1) {
	//cout << "gene " << i << " selected\n";
      child1.gene[i] = gene[i];
      avail1[gene[i]] = false;
      child2.gene[i] = otherparent.gene[i];
      avail2[otherparent.gene[i]] = false;
    }
    else {
      notdoneyet[nrofnotdone++] = i;
      avail1[gene[i]] = true;
      avail2[otherparent.gene[i]] = true;
    }
    mask >>= 1;
  }

  c1 = 0;
  c2 = 0;
  for (i = 0; i < nrofnotdone; i++) {
    while (!avail1[otherparent.gene[c1]]) c1++;
    child1.gene[notdoneyet[i]] = otherparent.gene[c1++];
    while (!avail2[gene[c2]]) c2++;
    child2.gene[notdoneyet[i]] = gene[c2++];
  }

  //child1.Print();
  //child2.Print();

} /* GenoType::CrossoverOrder2() */

void GenoType::CrossoverPMX(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int pos1[MAXGENES];
  int pos2[MAXGENES];
  bool avail[MAXGENES];
  int skipped[MAXGENES];
  int doubled[MAXGENES];
  int g1, g2, i, sc, dc;

  for (i = 0; i < nrofgenes; i++) {
    pos1[gene[i]] = pos2[otherparent.gene[i]] = i;
    avail[i] = true;
  }

  g1 = rnd.asLong() % nrofgenes;
  g2 = rnd.asLong() % nrofgenes;

  // child 1:
  for (i = g1; i != g2; i = (i + 1) % nrofgenes) {
    child1.gene[i] = gene[i];
    avail[i] = false;    
  }

  sc = dc = 0;
  for (i = g1; i != g2; i = (i + 1) % nrofgenes) {
    // elt in parent2 not used yet
    if (!(((g1 < g2) && (pos1[otherparent.gene[i]] < g2) && 
                       (pos1[otherparent.gene[i]] >= g1)) ||
         ((g1 > g2) && ((pos1[otherparent.gene[i]] >= g1) ||
                       (pos1[otherparent.gene[i]] < g2)))))
      if (avail[pos2[gene[i]]]) // position is free in offspring 1
      {  
        child1.gene[pos2[gene[i]]] = otherparent.gene[i];
        avail[pos2[gene[i]]] = false;
      }
      else {
	skipped[sc++] = otherparent.gene[i];  // position was used, skip tmp. 
      }
    else
      if (avail[pos2[gene[i]]]) {
        doubled[dc++] = pos2[gene[i]]; // fill again later  
      }
  }
  for (i = 0; i < sc; i++) {
    child1.gene[doubled[i]] = skipped[i];
    avail[doubled[i]] = false;
  }
  for (i = 0; i < nrofgenes; i++)
    if (avail[i])
      child1.gene[i] = otherparent.gene[i];
    else
      avail[i] = true;

  // child 2:
  for (i = g1; i != g2; i = (i + 1) % nrofgenes) {
    child2.gene[i] = otherparent.gene[i];
    avail[i] = false;    
  }

  sc = dc = 0;
  for (i = g1; i != g2; i = (i + 1) % nrofgenes) {
    if (!(((g1 < g2) && (pos2[gene[i]] < g2) && 
                       (pos2[gene[i]] >= g1)) ||
         ((g1 > g2) && ((pos2[gene[i]] >= g1) ||
                       (pos2[gene[i]] < g2)))))
      if (avail[pos1[otherparent.gene[i]]]) // position is free in offspring 1
      {  
        child2.gene[pos1[otherparent.gene[i]]] = gene[i];
        avail[pos1[otherparent.gene[i]]] = false;
      }
      else
	skipped[sc++] = gene[i];  // position was used, skip tmp.
    else
      if (avail[pos1[otherparent.gene[i]]])
        doubled[dc++] = pos1[otherparent.gene[i]]; // fill again later  
  }
  for (i = 0; i < sc; i++) {
    child2.gene[doubled[i]] = skipped[i];
    avail[doubled[i]] = false;
  }
  for (i = 0; i < nrofgenes; i++)
    if (avail[i])
      child2.gene[i] = gene[i];

} /* GenoType::CrossoverPMX() */

void GenoType::CrossoverCycle(GenoType& otherparent, GenoType& child1, GenoType& child2) {

  int pos1[MAXGENES];
  int pos2[MAXGENES];
  bool avail1[MAXGENES];
  bool avail2[MAXGENES];
  int g1, g2, i;

  for (i = 0; i < nrofgenes; i++) {
    pos1[gene[i]] = pos2[otherparent.gene[i]] = i;
    avail1[i] = avail2[i] = true;    
  }

  g1 = rnd.asLong() % nrofgenes;
  g2 = rnd.asLong() % nrofgenes;
  
  for (i = 0; avail1[g1] && (i < nrofgenes); i++) {
    child1.gene[g1] = gene[g1];
    avail1[g1] = false;
    g1 = pos2[gene[g1]];
  }
  for (i = 0; i < nrofgenes; i++)
    if (avail1[i])
      child1.gene[i] = otherparent.gene[i];


  for (i = 0; avail2[g2] && (i < nrofgenes); i++) {
    child2.gene[g2] = otherparent.gene[g2];
    avail2[g2] = false;
    g2 = pos1[otherparent.gene[g2]];
  }
  for (i = 0; i < nrofgenes; i++)
    if (avail2[i])
      child2.gene[i] = gene[i];

} /* GenoType::CrossoverCycle() */


#define MAXSIZE MAXPARENTS + 2

#define EdgeMapInsert(map, mapsize,  n1, n2) \
for (k = 0; (map[n1][k] != n2) && (k < mapsize[n1]); k++);\
  if (k == mapsize[n1])\
    map[n1][mapsize[n1]++] = n2;\
/* MACRO: EdgeMapInsert() */

#define EdgeMapRemove(n1) \
  for (k = 0; k < edgemapsizefrom[n1]; k++) {\
    temp = edgemapfrom[n1][k];\
    if (edgemapsize[temp] != MAXSIZE) {\
      for (l = 0; edgemap[temp][l] != n1; l++);\
      edgemapsize[temp]--;\
      for (; l < edgemapsize[temp]; l++)\
      edgemap[temp][l] = edgemap[temp][l+1];\
    }\
  }
/* MACRO: EdgeMapRemove() */


void GenoType::CrossoverEdge(int pi[], int parent1, GenoType pool[], 
                             int child1) {

  int i, j, k, l, temp;
  int edgemap[MAXGENES][MAXPARENTS];
  int edgemapfrom[MAXGENES][MAXPARENTS];
  int edgemapsize[MAXGENES];
  int edgemapsizefrom[MAXGENES];
  int minsize, nrofmin, nodenr, nextnodenr;
  Allel minima[MAXGENES];

  /* Create Edge Map */
  for (i = 0; i < nrofgenes; i++) { 
    edgemapsize[i] = 0;
    edgemapsizefrom[i] = 0;
  }
  
  for (i = 0; i < nrofparents; i++) {
      //cout << "parent " << i << ": ";
      //pool[pi[i]].Print();
    EdgeMapInsert(edgemap, edgemapsize, pool[pi[i]].gene[0], pool[pi[i]].gene[1]);
    EdgeMapInsert(edgemapfrom, edgemapsizefrom, pool[pi[i]].gene[0], pool[pi[i]].gene[nrofgenes-1]);
    for (j = 1; j < nrofgenes-1; j++) {
      EdgeMapInsert(edgemap, edgemapsize, pool[pi[i]].gene[j], pool[pi[i]].gene[j+1]);
      EdgeMapInsert(edgemapfrom, edgemapsizefrom, pool[pi[i]].gene[j], pool[pi[i]].gene[j-1]);
    }
    EdgeMapInsert(edgemapfrom, edgemapsizefrom, pool[pi[i]].gene[nrofgenes-1],pool[pi[i]].gene[nrofgenes-2]);
    EdgeMapInsert(edgemap, edgemapsize, pool[pi[i]].gene[nrofgenes-1],pool[pi[i]].gene[0]);
  }

  /* Print Edgemap 
  for (i = 0; i < nrofgenes; i++) {
    cout << i << ": ";
    for (j = 0; j < edgemapsize[i]; j++)
      cout << edgemap[i][j] << " ";
    cout << endl;
  }*/

  /* Choose initial node */
  minsize = MAXSIZE;
  nrofmin = 0;
  for (i = 0; i < nrofparents; i++)
    if (edgemapsize[pool[pi[i]].gene[0]] < minsize) {
      minima[0] = pool[pi[i]].gene[0];
      minsize = edgemapsize[minima[0]];
      nrofmin = 1;
    }
    else 
      if (edgemapsize[pool[pi[i]].gene[0]] == minsize)
        minima[nrofmin++] = pool[pi[i]].gene[0];
  if (nrofmin > 1)
    nodenr = minima[rnd.asLong() % nrofmin];
  else 
    nodenr = minima[0];

//  cout << "Initial node: " << nodenr << endl;
  //pool[child1].gene[0] = nodenr;

  /* Select node with edgemap and remove node from edgemap */
  for (i = 1; i < nrofgenes; i++) {  
    /* put selected node in permutation and remove it from edgemap */
    EdgeMapRemove(nodenr);
    /* select next node from neighbours */
    minsize = MAXSIZE-1;
    nrofmin = 0;
    for (j = 0; j < edgemapsize[nodenr]; j++)
      if (edgemapsize[edgemap[nodenr][j]] < minsize) {
        minima[0] = edgemap[nodenr][j];
        minsize = edgemapsize[minima[0]];
        nrofmin = 1;
      }
      else 
        if (edgemapsize[edgemap[nodenr][j]] == minsize)
          minima[nrofmin++] = edgemap[nodenr][j];

    edgemapsize[nodenr] = MAXSIZE;

    if (nrofmin == 1)
      nextnodenr = minima[0];
    else
      if (nrofmin > 1)
        nextnodenr = minima[rnd.asLong() % nrofmin];
      else {
	  //    cout << "NO CONTINUING EDGE\n";
        minsize = MAXSIZE - 1;
        nrofmin = 0;
        for (k = 0; k < nrofgenes; k++)
   	  if (edgemapsize[k] < minsize) {
            minima[0] = k;
            minsize = edgemapsize[k];
            nrofmin = 1;
	  } 
          else 
            if (edgemapsize[k] == minsize)
   	      minima[nrofmin++] = k;

        if (nrofmin > 1)
          nextnodenr = minima[rnd.asLong() % nrofmin];
        else
          nextnodenr = minima[0];
      }
    /* Print Edgemap 
    for (k = 0; k < nrofgenes; k++) {
      cout << k << ": ";
      if (edgemapsize[k] < MAXSIZE)
        for (l = 0; l < edgemapsize[k]; l++)
          cout << edgemap[k][l] << " ";
      cout << endl; 
    }*/
    nodenr = nextnodenr;
    pool[child1].gene[i] = nodenr;
  //cout << "Next node: " << nodenr << endl;
  }

//  cout << "Child: ";
//pool[child1].Print();

} /* GenoType::CrossoverEdge() */

/* Print  chromosome's genes and fitness value */
void GenoType::Print() {

  int i;

  for (i = 0; i < nrofgenes; i++)
    cout << gene[i] << " ";
  cout << ", ";

  cout << "fitness: " << fitness << endl;

} /* GenoType::Print() */

 
class Population {
public:
  GenoType pool[MAXPOPSIZE + MAXOFFSPRING * OPUSPERGEN]; // pool of chromosomes
  double fitnesssum;             // total fitness of population
  int bestindex;                 // index of chromosome with best fitness
  int difthresh;                 // difference threshold for incest prevention
  double genevariance[MAXGENES]; // variance of alleles over whole population
  double genevaravg;
  double geneentropy[MAXGENES];  // entropy of each gene
  double geneentavg;             // average entropy of all genes
  double fitvariance;            // variance in fitness over population
  double fitdifference;          // difference between best and worst fitness
  bool converged;
  void Initialize();
  void Evaluate();
  void SelectParentsAndReproduce();
  int RemoveIndividual(int);  
  void SelectSurvivors();
  void ComputeConvergence(long&);
  void PrintBest();
  void PrintPop();
private:
  bool firstconverged;
  long firstconveval;
  double firstconvvalue;
  double fitnesssqrsum;
  int allelsum[MAXGENES];
  long allelsqrsum[MAXGENES];
  int alleloccurence[MAXGENES][NROFCOLORS];
  void SelectParents(int[]);
  void AdjustStatistics(int);
};


/* Initialize all chromosomes in the population */
void Population::Initialize() {

  int i, j;

  if (INTREP)
    for (i = 0; i < popsize; i++)
      pool[i].IntegerInitialize();
  else 
    for (i = 0; i < popsize; i++)
      pool[i].OrderInitialize();

  difthresh = nrofgenes * (NROFCOLORS - 1) /  NROFCOLORS;
  converged = false;
  geneentavg = 0;
  fitnesssqrsum = 0;
  fitvariance = 0;
  fitdifference = 0;
  firstconveval = 0;
  firstconvvalue = 0;
  firstconverged = false;  

  for (i = 0; i < nrofgenes; i++) {
    problem.weight[i] = 1;
    genevariance[i] = 0; 
    allelsum[i] = 0;
    allelsqrsum[i] = 0;
    for (j = 0; j < NROFCOLORS; j++)
      alleloccurence[i][j] = 0;
  }

} /* Population::Initialize() */

/* Evaluate the population and compute other statistics */
void Population::Evaluate() {

  int i;

  bestindex = 0;
  fitnesssum = 0;

  if (INTREP)
    for (i = 0; i < popsize; i++) {
      pool[i].IntegerEvaluate();
      AdjustStatistics(i);
    } 
  else 
    for (i = 0; i < popsize; i++) {
      pool[i].OrderEvaluate();
      AdjustStatistics(i);
    }  
  
} /* Population::Evaluate() */

/* Adjust Statistics when chromosome with index 'index' has been added. */
void Population::AdjustStatistics(int index) {

  int i;
  Allel allel;

  /* Check if bestfitness of population changes and adjust fitnesssum */
  if (pool[index].fitness < pool[bestindex].fitness)
    bestindex = index;

  fitnesssum += pool[index].fitness;

  if (STOPCRIT == fitvarconv || STOPCRIT == allconv)
    fitnesssqrsum += pool[index].fitness * pool[index].fitness;
  
  if (STOPCRIT == genevarconv || STOPCRIT == allconv)
    for (i = 0; i < nrofgenes; i++) {
      allel = pool[index].gene[i];
      allelsum[i] += allel;
      allelsqrsum[i] += allel * allel;
    }
  if (STOPCRIT == entropyconv || STOPCRIT == allconv)
    for (i = 0; i < nrofgenes; i++)
      alleloccurence[i][pool[index].gene[i]]++;

} /* Population::AdjustStatistics() */


/* Select 'nrofparents' (not necessarily distinct) parents with tournament */
/* selection. The chance for equal parents is low enough...                */
/* OUT: pi[] - indices of the 'nrofparents' parents in the population.     */
void Population::SelectParents(int pi[]) {

  register int i;
  register int j;
  int winner, competitor;
  int fails = 0;
  double dist;
  register double factor;
  Allel allel;
 
  do {
    for (i = 0; i < (nrofparents * OPUSPERGEN); i++) {
      /* Do a tournament between TOURNSIZE chromosomes.                   */
      /* the set of TOURNSIZE competitors is not disjunct (at the moment) */
      winner = rnd.asLong() % popsize;
      for (j = 1; j < TOURNSIZE; j++) {
        competitor = rnd.asLong() % popsize;
        if (pool[competitor].fitness < pool[winner].fitness)
          winner = competitor;
      }
      pi[i] = winner;
    }
    dist = 0;
    if (INCESTPREV) {
      if (true) {
        for (i = 0; (i < nrofgenes) && (dist < difthresh); i++) {
          allel = pool[pi[0]].gene[i];           
          for (j = 1; (j < nrofparents)&&(pool[pi[j]].gene[i] == allel); j++);
          if (j < nrofparents)
            dist++;
        }
	/*  register int isum;
        register int isqrsum;

        for (i = 0; i < nrofparents; i++)
          parentgene[i] = pool[pi[i]].gene;
        factor = 1 / (double) nrofparents;
        for (i = 0; (i < nrofgenes) && (dist < difthresh); i++) {
          isum = 0; isqrsum = 0;
          for (j = 0; j < nrofparents; j++) {
            allel = pool[pi[j]].gene[i];
            isum += allel; 
            isqrsum += (allel * allel); 
          }
          //if ((sqrsum - (sum * sum)/nrofparents)/nrofparents > 0)
          //   dist++; 
          dist += ((isqrsum - isum*isum*factor) * factor);  
        } */
      }
      else {
        register double colorlog;
        register double sum;
        register double fraction;
        int freq[NROFCOLORS];

        factor = 1 / (double) nrofparents;
        colorlog = 1 / log((double)NROFCOLORS);
        for (i = 0; (i < nrofgenes) && (dist < difthresh); i++) {
          for (j = 0; j < NROFCOLORS; j++)
            freq[j] = 0;
          for (j = 0; j < nrofparents; j++)
            freq[pool[pi[j]].gene[i]]++;
          sum = 0;
          for (j = 0; j < NROFCOLORS; j++)
	    if (freq[j]) {
              fraction = freq[j] * factor;
              sum -= (fraction * log(fraction));
	    }
          dist += (sum * colorlog);
          //if ((sum * colorlog) > 0.7)
	  // dist++; 
	}
      }
      //  cout  << "dist: " << dist << endl;
      if (dist < difthresh) {
        fails++;
        if (fails == NROFFAILS) {
          fails = 0;
          difthresh--;
	  //    cout << "difthresh: " << difthresh << endl; 
        }  
      }
    } 
  } while (INCESTPREV && (dist < difthresh));

} /* Population::SelectParents() */


void Population::SelectParentsAndReproduce() {

  int i;
  int parents[MAXPARENTS * OPUSPERGEN]; //  indices of the parents 

  SelectParents(parents);

  /* reproduce parents and add offspring at end of pool */
  for (i = 0; i < (nrofoffspring * OPUSPERGEN); i += nrofoffspring) 
    switch (crossover) {
      case edge: pool[parents[i]].CrossoverEdge(parents, i, pool, popsize + i);
                 break;
      case order:pool[parents[i]].CrossoverOrder(pool[parents[i+1]], 
                 pool[popsize + i], pool[popsize + i + 1]); 
                 break;
      case order2:pool[parents[i]].CrossoverOrder2(pool[parents[i+1]], 
                  pool[popsize + i], pool[popsize + i + 1]); 
                  break;
      case cycle:pool[parents[i]].CrossoverCycle(pool[parents[i+1]], 
                  pool[popsize + i], pool[popsize + i + 1]); 
                  break;
      case pmx:pool[parents[i]].CrossoverPMX(pool[parents[i+1]], 
                  pool[popsize + i], pool[popsize + i + 1]); 
                  break;
      case diag: pool[parents[i]].CrossoverDIAGMP(parents, i, 
                                  pool, popsize + i);
                 break;
      case scun: pool[parents[i]].CrossoverSCUNMP(parents, i, 
                                         pool, popsize + i);
                        break;
      case hux: pool[parents[i]].HUX(pool[parents[i+1]], 
                     pool[popsize + i], pool[popsize + i + 1]);
                break;
      case uniform: pool[parents[i]].CrossoverU(pool[parents[i+1]], 
                         pool[popsize + i], pool[popsize + i + 1]);
                    break;
      case twopoint: pool[parents[i]].Crossover2(pool[parents[i+1]], 
                          pool[popsize + i], pool[popsize + i + 1]);
                     break;
      case onepoint: pool[parents[i]].Crossover1(pool[parents[i+1]], 
                          pool[popsize + i], pool[popsize + i + 1]);
	             break;
      case mpoint: pool[parents[i]].CrossoverM(pool[parents[i+1]], 
                        pool[popsize + i], pool[popsize + i + 1]);
                   break;
      case noop: pool[parents[i]].Copy(pool[popsize + i]);
                 break;
      default: cout << "Error: no CROSSOVER defined!" << endl;
    } 
   
    /* Mutate and evaluate offspring */
    for (i = 0; i < (nrofoffspring * OPUSPERGEN); i++) {
      switch (MUTATION) {
        case intmut: pool[popsize + i].MutateInteger();
                      break;
        case swap: pool[popsize + i].MutateSwap();
                      break;
        case rar: pool[popsize + i].MutateRAR();
                  break;
/*      case inversion: pool[popsize + i].MutateInversion();
                        break;
        case heur: pool[popsize + i].MutateHeur();
                   break;
        case normal: pool[popsize + i].MutateNormal();
                      break; 
        //  WEG OMDAT OPTIMALISATIE ANDERS NIET WERKT */
        case noop: break;
        default: cout << "Error: no MUTATION defined!" << endl;
      }
      // For Integer Representation no evaluation is necessary anymore as
      // SetAllel has locally recalculated the fitness already 
      if (!INTREP)
        pool[popsize+i].OrderEvaluate();
      AdjustStatistics(popsize + i); 
    }   

} /* Population::SelectParentsAndReproduce() */

/* Remove the leastfit individual from the population, by emptying */
/* its position, readjusting the statistics and returnin the index */
/* of the position of the leastfit individual.                     */
/* IN: ps - the current population size                            */
int Population::RemoveIndividual(int ps) {

  int i,
      start,
      leastfit;
  Allel allel;

  /* SLOW: Search each time whole population to find least fit chromosome */
  start = rnd.asLong() % popsize; 
  leastfit = start;
  if ((fitnesssum*invpopsize) - pool[bestindex].fitness != 0) 
    for (i = start+1; i != (start+ps)%ps; i = (i + 1) % ps)
      if (pool[i].fitness > pool[leastfit].fitness) 
        leastfit = i;

  /* Adjust Statistics */
  fitnesssum -= pool[leastfit].fitness;

  if (STOPCRIT == fitvarconv || STOPCRIT == allconv)
    fitnesssqrsum -= pool[leastfit].fitness * pool[leastfit].fitness;

  if (STOPCRIT == genevarconv || STOPCRIT == allconv)
    for (i = 0; i < nrofgenes; i++) {
      allel = pool[leastfit].gene[i];
      allelsum[i] -= allel;
      allelsqrsum[i] -= allel * allel;
    }

  if (STOPCRIT == entropyconv || STOPCRIT == allconv)
    for (i = 0; i < nrofgenes; i++)
      alleloccurence[i][pool[leastfit].gene[i]]--;

  return leastfit;

} /* Population::RemoveIndividual() */


#define INVALID -1

/* Select popsize survivors for next generation by removing the */
/* OPUSPERGEN * nrofoffspring  chromosomes with least fitness.  */
/* CAN BE DONE FASTER!!!!                                       */
void Population::SelectSurvivors() {

  int ps;           /* size of population with the remaining children */


  if (PRESERVATIVE) {
    int leastfit;   /* chromosome with worst fitness in united population */

    /* Select 'nrofoffspring * OPUSPERGEN' least fit chromosomes */
    for (ps = popsize + (nrofoffspring * OPUSPERGEN); ps > popsize; ps--) {
      leastfit = RemoveIndividual(ps);
 
      /* replace leastfit with a child */
      pool[ps-1].Copy(pool[leastfit]);
      if (bestindex == ps-1)
        bestindex = leastfit;
    }
  }
  else {            
    int i, start, bestfit;
    
    bestindex = 0;
    fitnesssum = 0;
    for (ps = 0; ps < popsize; ps++) {
      start = rnd.asLong() % (nrofoffspring * OPUSPERGEN); 
      do start = (start+1) % (nrofoffspring * OPUSPERGEN);
      while (pool[popsize+start].fitness == INVALID);
      bestfit = popsize+start;
      for (i = ((start+1)%(nrofoffspring * OPUSPERGEN)); 
           i != start; i = (i + 1) % (nrofoffspring * OPUSPERGEN)) {
        if ((pool[popsize+i].fitness < pool[bestfit].fitness) &&
            (pool[popsize+i].fitness != INVALID))
          bestfit = popsize+i;
      }
      // Add offspring to population
      pool[bestfit].Copy(pool[ps]);
      pool[bestfit].fitness = INVALID;
      if (pool[ps].fitness < pool[bestindex].fitness)
        bestindex = ps;     
      fitnesssum += pool[ps].fitness;
    }
  }
    

} /* Population::SelectSurvivors() */

/* Compute the variance for each gene over all the individuals of the */
/* population, using precalculated statistics.                        */
void Population::ComputeConvergence(long &evals) {

  int i, j;

  converged = false;
  
  switch (STOPCRIT) {
    case noconv: break;
    case allconv:
    case fitdifconv: fitdifference = ((fitnesssum/popsize) - 
                                      pool[bestindex].fitness); 
	             if (fitdifference == 0) {
	               if (!firstconverged) {
                         firstconverged = true;
                         firstconveval = evals;
                       }
	               if (evals > firstconveval + MAXCONVTIME)
                         converged = true;
                     }
		     else 
                       firstconverged = false;
                     if (!(STOPCRIT == allconv)) break;
    case fitvarconv: fitvariance = ((fitnesssqrsum - (fitnesssum*fitnesssum / 
                                   popsize)) / popsize);
                     converged = converged || (fitvariance == 0); 
                     if (!(STOPCRIT == allconv)) break;
    case genevarconv: genevaravg = 0; 
                      for (i = 0; i < nrofgenes; i++) {
                        genevariance[i] = ((double) allelsqrsum[i] - ((double)
                              (allelsum[i] * allelsum[i])/popsize)) / popsize;
                        genevaravg += genevariance[i];
                      }
                      genevaravg /= nrofgenes;
                      converged = converged || (genevaravg == 0);
                      if (!(STOPCRIT == allconv)) break;
    case entropyconv: geneentavg = 0;
          	      for (i = 0; i < nrofgenes; i++) {
                        geneentropy[i] = 0;
                        for (j = 0; j < NROFCOLORS; j++)
			  if (alleloccurence[i][j] > 0)
                            geneentropy[i] += (-((double)alleloccurence[i][j]
                                              /popsize)*
                                              log((double)alleloccurence[i][j]
                                              /popsize));
                        geneentropy[i] /= log((double) NROFCOLORS);
                        geneentavg += geneentropy[i];
                      }
                      geneentavg /= nrofgenes;
                      converged = converged || (geneentavg == 0);
                      if (!(STOPCRIT == allconv)) break;
  }

} /* Population::ComputeConvergence() */


/* print all chromosomes of population */
void Population::PrintPop() {

  int i,j;

  for(i = 0; i < popsize; i++) {
    cout << "c" << i << ":";
    for (j = 0; j < nrofgenes; j++)
      cout << " " << pool[i].gene[j];   
    cout << ", fitness: " << pool[i].fitness << endl;
  }
} /* Population::PrintPop() */  

/* class to store statistics about solutions that where found */
class ExpRunStats {
public:
  long evalnr;             // nr of evaluations x 
  double successrate;      // rate of successes for x evaluations 
  int expruns;             // expected nr of runs for x evaluations 
                           // to find at least one solution.        
  long expevals;           // exp. total nr of evaluations to find solution 
};


class Ga {
public:
  Population pop;
  int runnr;
  long evals;
  int successes;             // number of solutions found 
  void Initialize();
  bool Finished();
  void Run();
  void RemoveTempReportFiles();
  void MakeTotalReport(ofstream&);
private:
  void ChangeWeights();
  ExpRunStats ers[MAXRUNS];  // statistics to find expected needed nr of runs
  double bestfiteval[MAXEVALS/WRITEFILEINTERVAL+WRITEFILEINTERVAL];
  double bestfitsum[MAXEVALS/WRITEFILEINTERVAL+WRITEFILEINTERVAL];
  double bestfitsqrsum[MAXEVALS/WRITEFILEINTERVAL+WRITEFILEINTERVAL];
  long maxlastwrite;
  long nextwrite;
  long nextwghtinc;
  int nrofwrite;
  ofstream output;  
  void WriteTempResult();
};

void Ga::Initialize() {

  int i;
  char Filename[MAXPATHLENGTH];  

  runnr = 0;
  CreateGenMutProbs();
  longsize = sizeof(long);
  successes = 0;
  maxlastwrite = 0;
  invpopsize = 1.0/popsize;

  for (i = 0; i < MAXEVALS/WRITEFILEINTERVAL + WRITEFILEINTERVAL; i++) {
    bestfitsum[i] = 0;
    bestfitsqrsum[i] = 0;
  }

  strcpy(Filename, RESULTDIR);
  strcat(Filename, problem.ProblemFilename);
  strcat(Filename, methodname);
  strcat(Filename, ".tmp");
  output.open(Filename);

} /* Ga:: Initialize() */

bool Ga::Finished() {

  return ((evals > MAXEVALS) ||
          (pop.pool[pop.bestindex].fitness == 0) ||
          (STOPCRIT==fitdifconv && pop.converged) ||
          (STOPCRIT==fitvarconv && pop.converged) ||
          (STOPCRIT==genevarconv && pop.converged) ||
          (STOPCRIT==entropyconv && pop.converged) ||
          (STOPCRIT==allconv && pop.converged));

} /* Ga::Finished() */

void Ga::ChangeWeights() {

  if (INTREP) {
    int i;
    Link *ptr;
    Allel gene[MAXGENES];
  
    gene = pop.pool[pop.bestindex].gene;
  
    for (i = 0; i < nrofgenes; i++)
      for (ptr = problem.gradjl[i]; ptr; ptr = ptr->nextptr)
        if (gene[i] == gene[ptr->nodenr]) 
          ptr->weight += WGHTINC;    
  }
  else
  {
    int color[MAXGENES];
    int i, j;
    int index;
    bool avail[MAXGENES][NROFCOLORS];
    Link* ptr;

    // Initialize 
    for (i = 0; i < nrofgenes; i++) {
      color[i] = UNCOLORED;
      for (j = 0; j < NROFCOLORS; j++)
        avail[i][j] = true;
    }

    // Color all nodes 
    for (i = 0; i < nrofgenes; i++) {
      index = pop.pool[pop.bestindex].gene[i];
      for (j = 0; (j < NROFCOLORS) && !avail[index][j]; j++);
      if (j < NROFCOLORS) {
        color[index] = j;

        for (ptr = problem.gradjl[index]; ptr; ptr = ptr->nextptr)
          if (color[ptr->nodenr] == UNCOLORED)
            avail[ptr->nodenr][j] = false;
      }
      else
           problem.weight[index] += WGHTINC;
    }
  }

} /* Ga::ChangeWeights() */
 
/* Performs a run of the GA for the specified problem and writes */
/* the results to a file with name <problemfilename>.res<runnr>    */
void Ga::Run() {

  int i; 

  runnr++;
  
  StartTimer(); 

  rnd.seed1(0);
  rnd.seed2(runnr);

  pop.Initialize();
  pop.Evaluate();
//  pop.ComputeConvergence(evals);

  evals = popsize;
  nextwrite = popsize;
  nextwghtinc = WGHTCHEVAL;
  nrofwrite = 0;
  while (!Finished()) {
//    pop.ComputeConvergence(evals);
    WriteTempResult();  
    pop.SelectParentsAndReproduce();
    pop.SelectSurvivors(); 
    evals += nrofoffspring * OPUSPERGEN;
    if ((WEIGHTS == sawwghts) && (evals >= nextwghtinc)) {
      ChangeWeights(); 
      pop.Evaluate();
      nextwghtinc += WGHTCHEVAL;
    }
  }
  nextwrite = 0;
  WriteTempResult();
  if (nrofwrite > maxlastwrite)
    maxlastwrite = nrofwrite-1;

  StopTimer();
  totalevaltime += ((double) GetRunTime() / evals);
       
  output << "Run " << runnr << ": ";
  if (pop.pool[pop.bestindex].fitness == 0) {
    i = successes;
    while ((i > 0) && (evals < ers[i-1].evalnr)) {
      ers[i].evalnr = ers[i-1].evalnr;
      i--;
    }
    ers[i].evalnr = evals;
    successes++;
    output << "Solution found after " << evals << " evaluations!" << endl;
    // Print the solution found to the screen
    //pop.pool[pop.bestindex].Print();
    
  }
  else {
    output << "No Solution found after " << evals << " evaluations." << endl;
    output << "Best fitness: " << pop.pool[pop.bestindex].fitness << 
              ", Avg fitness: " << pop.fitnesssum / popsize << endl; 
  }
  cout << endl << endl;
    
} /* Ga::Run() */


/* Write temporary results to reportfile */
void Ga::WriteTempResult() {
  
  if (evals >= nextwrite) {
      cout << evals << " " << pop.pool[pop.bestindex].fitness << " " 
           << pop.fitnesssum / popsize << endl;
                   
    bestfiteval[nrofwrite] = evals;
    bestfitsum[nrofwrite] += pop.pool[pop.bestindex].fitness;
    bestfitsqrsum[nrofwrite] += ((pop.pool[pop.bestindex].fitness) * 
                                 (pop.pool[pop.bestindex].fitness));
    nrofwrite++;
    nextwrite += WRITEFILEINTERVAL;
  }
  
} /* Ga::WriteTempResult() */

void Ga::RemoveTempReportFiles() {

  char string[MAXPATHLENGTH]; 
   
  strcpy(string, "rm ");
  strcat(string, RESULTDIR);
  strcat(string, problem.ProblemFilename);
  strcat(string, methodname);
  strcat(string, ".tmp");
  system(string);

  strcpy(string, "rm ");
  strcat(string, RESULTDIR);
  strcat(string, problem.ProblemFilename);
  strcat(string, methodname);
  strcat(string, ".sdres");
  system(string);

  strcpy(string, "rm ");
  strcat(string, RESULTDIR);
  strcat(string, problem.ProblemFilename);
  strcat(string, methodname);
  strcat(string, ".avgres");
  system(string);

} /* Ga::RemoveTempReportFiles() */



/* Compute following statistics over all runs for this problem */
/* for run 1 to current runnr.                               */
/* - average bestfitness over all runs for each evaluationnr */
/* - standard deviation for average fitness                  */
/* (- worst fitness of all runs for each evaluationnr)       */
/* (- best fitness of all runs for each evaluationnr)        */
/* - success rate over all runs                              */
/* - average nr of evaluations to find the solution if found */
/* - stdev of evaluations to find the solution if found.     */
/* write results to file in gnuplot format.                  */
void Ga::MakeTotalReport(ofstream &reportoutput) {

  int i;                    /* number of the run */
  char AvgOutputFilename[MAXPATHLENGTH];
  char SDOutputFilename[MAXPATHLENGTH];
  double avgeval;           /* average nr of evaluations to find solution */
  int nrofruns;

  long minexpevals;
  double sumerrorsqr; 
  ofstream sdoutput;
  ofstream avgoutput;

  strcpy (AvgOutputFilename, RESULTDIR);
  strcat (AvgOutputFilename, problem.ProblemFilename);
  strcat (AvgOutputFilename, methodname);
  strcpy (SDOutputFilename, AvgOutputFilename);
  strcat (AvgOutputFilename, ".avgres");
  strcat (SDOutputFilename, ".sdres");
    
  avgoutput.open(AvgOutputFilename);
  sdoutput.open(SDOutputFilename);
  nrofruns = NROFRUNS;

  for (i = 0; i <= maxlastwrite; i++) {
    avgoutput<<bestfiteval[i]<<" "<<(bestfitsum[i]/((double)nrofruns))<<endl;
    if (nrofruns > 1)
      sdoutput <<bestfiteval[i]<<" "<< sqrt((bestfitsqrsum[i]-(bestfitsum[i]*
               bestfitsum[i]/((double) nrofruns)))/((double)nrofruns-1))<<endl;
    else
      sdoutput << bestfiteval[i] << " -0" << endl;
  }
  
  avgeval = 0;
  for (i = 0; i < successes; i++) {
    ers[i].successrate = (double)(i+1)/(double) nrofruns;
    if (ers[i].successrate < 1.0)
      ers[i].expruns = (int) ceil (log(1.0 - EFECERTAINTY) / 
                               log(1.0 - ers[i].successrate));
    else
      ers[i].expruns = 1;
    ers[i].expevals = ers[i].expruns * ers[i].evalnr;
    avgeval += ers[i].evalnr;
  }
  avgeval /= successes;

  /* Succes rate, avg best fitness of last evaluation over all */
  /* runs and Standard Deviation over these runs.              */
  reportoutput << (double) successes / nrofruns << ", "
               << (bestfitsum[maxlastwrite]/((double)nrofruns)) << ", ";
  if (nrofruns > 1) 
    reportoutput << 
                sqrt((bestfitsqrsum[maxlastwrite]-(bestfitsum[maxlastwrite]*
                bestfitsum[maxlastwrite]/((double) nrofruns)))/(nrofruns-1));
  else
    reportoutput << "-0";
  
  /* Compute stdev for nr of evaluations needed to find solution */
  /* this is over runs with solution.                            */
  sumerrorsqr = 0;
  for (i = 0; i < successes; i++) 
    if (ers[i].evalnr > 0) 
      sumerrorsqr += ((ers[i].evalnr - avgeval) * (ers[i].evalnr - avgeval));
  reportoutput << ", " << avgeval << ", ";
  if (nrofruns > 1)
    reportoutput << sqrt(sumerrorsqr / (double) (successes-1)) << ", ";
  else
    reportoutput << "-0" << ", ";

  /* Output expected nrs of evalations to find at least one solution */
  avgoutput << "# evalnr: successrate: expruns: expevals: " << endl;
  if (successes > 0)
    minexpevals = ers[0].expevals;
  else
    minexpevals = 0;
  for (i = 0; i < successes; i++) {
    avgoutput << "# " << ers[i].evalnr << " " << ers[i].successrate << " " 
              << ers[i].expruns << " " << ers[i].expevals << endl;
    if (ers[i].expevals < minexpevals)
      minexpevals = ers[i].expevals;
  }
  avgoutput << "# minimal expected nr of evaluations to find at least "
            << "one solution is " << minexpevals << "." << endl;
  reportoutput << minexpevals << ", ";
  
  reportoutput << (totalevaltime / nrofruns) * 1000 << ", ";
  reportoutput << (minexpevals * (totalevaltime / nrofruns)) << endl;

  avgoutput.close();
  sdoutput.close();
  output.close();
   
} /* Ga::MakeTotalReport() */

/* Do 'NROFRUNS' runs for the specified operator */
/* and setup things necessary for the operator   */
void GaRunOperator(Operator op, int nrofp, int nrofo) {

  Ga ga;
  char opname[MAXPATHLENGTH];
  int digit;
  int r, l;

  crossover = op;
  nrofparents = nrofp;
  nrofoffspring = nrofo;

  switch (op) {
    case onepoint: strcpy (opname, "1point"); break;
    case twopoint: strcpy (opname, "2point"); break;
    case uniform: strcpy (opname, "uniform"); break;
    case hux: strcpy(opname, "hux"); break;
    case mpoint: strcpy (opname, "M");
                 l = strlen(opname);
                 nrofp = crosspoints;
                 for (digit = (int) log10(crosspoints); digit >= 0; digit--) {
                   opname[l++] = (nrofp/(int) pow(10.0, (double) digit))+'0';
                   nrofp = nrofp % (int) pow(10.0, (double) digit);
                 }
                 opname[l] = (char) NULL;
                 break;
    case diag: strcpy (opname, "DIAGMP");
               l = strlen(opname);
               for (digit=(int)log10(nrofparents); digit>=0; digit--) {
                 opname[l++]=(nrofp/(int)pow(10.0,(double) digit))+'0';
                 nrofp = nrofp % (int) pow(10.0, (double) digit);
               }
               opname[l] = (char) NULL;
               break;
    case scun: strcpy (opname, "SCUNMP");
               l = strlen(opname);
               for (digit=(int)log10(nrofparents); digit>=0; digit--) {
                 opname[l++]=(nrofp/(int)pow(10.0,(double) digit))+'0';
                 nrofp = nrofp % (int) pow(10.0, (double) digit);
               }
               opname[l] = (char) NULL;
               break;
    case edge: strcpy (opname, "EX"); break;
    case order: strcpy (opname, "OX"); break;
    case order2: strcpy (opname, "OX2"); break;
    case cycle: strcpy (opname, "Cycle"); break;
    case pmx: strcpy (opname, "PMX"); break;
    case noop: strcpy (opname, "nocross"); break;
    default: strcpy (opname, "ERROR"); cout<<"No Crossover defined!\n"; break;
  }
  
  reportoutput << opname << ": ";

/*  cout << opname << endl; */

  strcpy (methodname, FILELABEL);
  strcat (methodname, opname);

  ga.Initialize();

  totalevaltime = 0;
  for (r = 1; r <= NROFRUNS; r++) 
    ga.Run();

  ga.MakeTotalReport(reportoutput);

  if (REMOVETMPFILES) 
    ga.RemoveTempReportFiles(); 

} /* RunOperator() */

// Print specified parameter to report file
void PrintParameters(ofstream &reportoutput) {

  reportoutput << "Problem: " << problem.ProblemFilename << endl
               << "Number of runs: " << NROFRUNS << endl
               << "Maximum number of evaluations: " << MAXEVALS << endl
               << "Population size: " << POPSIZEFROM;
  if (POPSIZEFROM == MAXPOPSIZE) 
    reportoutput << endl;
  else
    reportoutput << " .. " << MAXPOPSIZE << endl;
  reportoutput << "Tournament size: " << TOURNSIZE << endl
               << "Operations per generation: " << OPUSPERGEN << endl
               << "Mutation: ";
  switch (MUTATION) {
    case heur: reportoutput << "heuristic mutation, randompos, heurval"<<endl;
               break;
    case swap: reportoutput << "swap mutation, order-based"<<endl;
               break;
    case rar: reportoutput << "RAR mutation, order-based"<<endl;
               break;
    case inversion: reportoutput << "inversion mutation, order-based"<<endl;
                    break; 
    case intmut: reportoutput << "fast integer mutation, 3 colors" << endl; 
                 break;
    case normal: reportoutput << "normal mutation, 3 colors" << endl; break;
    case noop: reportoutput << "no mutation!" << endl; break;
    default: reportoutput << "unknown mutation!" << endl; 
  }
  if (MR)
    reportoutput << "Mutation Rate: " << MR << endl;
  else
    reportoutput << "Mutation Rate: " << 1.0 / (double) N << " (1/L)\n";
  switch(WEIGHTS) {
    case nowghts: reportoutput << "Weights: Equal (1)" << endl;
                  break;
    case degreewghts: reportoutput << "Weights: Degree" << endl;
                      break;
    case sawwghts: reportoutput << "Weights: Stepwise Adapting";
                   reportoutput << ",T = " << WGHTCHEVAL 
                                << ", W_inc = " << WGHTINC << endl;
                   break;
                   break;
    default: reportoutput << "Weights: No Weight Mechanism Defined!" << endl;
  }
  if (PRESERVATIVE)
    reportoutput << "Selection: Preservative (" << POPSIZEFROM << "+" <<
                    MAXOFFSPRING * OPUSPERGEN << ")" << endl;
  else
    reportoutput << "Selection: Extinctive (" << POPSIZEFROM << "," <<
                    MAXOFFSPRING * OPUSPERGEN << ")" << endl;

  /* reportoutput << "StopCriterium: Fitness Difference Criterium, MaxEval or solution found." << endl; */
  if (INCESTPREV)
    reportoutput << "remark: Incest prevention used!" << endl; 

  reportoutput << endl << "-------------------------------RESULTS PER CROSSOVER---------------------------" << endl;

  reportoutput << "SR = Succes Rate" << endl
     << "AFLE = Avg. Fitness of Last Evaluation (Avg. over all runs)" << endl
     << "SFLE = Standard Deviation of AFLE" << endl
     << "AES = Avg. Evaluation nr. when GA found solution (Avg. over runs with solution)" << endl
     << "SES = Standard Deviation of AES" << endl 
     << "EFE = Minimal Expected nr of Evaluations to find a solution with " 
     << EFECERTAINTY * 100 << "\% certainty" << endl 
     << "ATE = Average Time per Evaluation (in seconds/1000 evaluations)" << endl
     << "EFEt = Minimal Expected Time to find a solution with "
     << EFECERTAINTY * 100 << "\% certainty" << endl
     << "-------------------------------------------------------------------------------"<< endl;
  reportoutput << "Operator: SR, AFLE,  SFLE,    AES,     SES,     EFE,    ATE,     EFEt" << endl
     << "-------------------------------------------------------------------------------"<< endl;
} /* PrintParameters() */

/* Test several operators for some parameters and make a report */
void TestOperators(int n, double p, int s) {

  char reportname[MAXPATHLENGTH];
  int nop;

  problem.CreateProblem(n, p, s);
  problem.ReadProblem(nrofgenes);

  strcpy(reportname, RESULTDIR);
  strcat(reportname, problem.ProblemFilename);
  strcat(reportname, FILELABEL);
  strcat(reportname, ".opreport");
  reportoutput.open(reportname); 

  popsize = MAXPOPSIZE;
 
  PrintParameters(reportoutput);

  if (false) {
      //GaRunOperator(onepoint, 2, 2);   

    for (nop = 1; nop <= MAXCROSSPOINTS; nop++) {
      crosspoints = nop;
      GaRunOperator(mpoint, 2, 2);
    }
  }
  else {
    crosspoints = MAXCROSSPOINTS;
    GaRunOperator(OVERCROSSOVER, MAXPARENTS, MAXOFFSPRING);
  }
  
  problem.RemoveProblem();
  
  reportoutput.close();   

} /* TestOperators() */

/* Test several operators for some parameters and make a report */
void TestOperator(int n, int s) {

  char reportname[MAXPATHLENGTH];
  double p;

  strcpy(reportname, RESULTDIR);
  strcat(reportname, FILELABEL);
  strcat(reportname, ".report");
  reportoutput.open(reportname); 

  PrintParameters(reportoutput);

  for (p = PFROM; p < PTO + (PSTEP*0.5); p += PSTEP) {
  problem.CreateProblem(n, p, s);
    problem.ReadProblem(nrofgenes);
    reportoutput << problem.ProblemFilename << ": " << endl;
    for (popsize = POPSIZEFROM; popsize <= MAXPOPSIZE; popsize += POPSIZESTEP){
      if (POPSIZEFROM != MAXPOPSIZE)
        reportoutput << popsize << ": ";
                
      crosspoints = MAXCROSSPOINTS;
      GaRunOperator(OVERCROSSOVER, MAXPARENTS, MAXOFFSPRING);
    }
    problem.RemoveProblem();
  }  
  reportoutput.close();   

} /* TestOperator() */



main() {

  // TestOperators(1250, 8.0/1250, 5);

  if (N <= MAXGENES)
    if (NROFRUNS < MAXRUNS)
      TestOperator(N, RNDSEED); 
    else
      cout << "No more than " << MAXRUNS << " runs allowed!" << endl;
  else
    cout << "No more than " << MAXGENES << " variables allowed!" << endl;
 
} /* main() */
